/**
  *  \file client/dialogs/ionforecast.cc
  *  \brief Ion Storm Forecast
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cmath>
#include "client/dialogs/ionforecast.h"
#include "game/formula.h"
#include "game/ion.h"
#include "util/nls.h"
#include "game/global.h"
#include "ui/uiglobal.h"
#include "ui/window.h"
#include "ui/layout.h"
#include "cpluslib/holder.h"
#include "gfx/palette.h"
#include "ui/widgets/framegroup.h"
#include "client/wscanchart.h"
#include "client/widgets/scanresult.h"
#include "ui/widgets/button.h"
#include "ui/prefix.h"
#include "client/widgets/helpwidget.h"
#include "ui/widgets/quit.h"

namespace {
    class WIonForecastChart : public WScannerChartWidget {
     public:
        WIonForecastChart()
            { }

        void drawPre(GfxCanvas& can);
        void setInitialZoom();
        bool handleEvent(const UIEvent& e, bool second_pass);
    };

    int getPredictionColor(const GIonStorm& ion, int prob)
    {
        // chartdlg.pas::SetStormColor; only _col256 branch
        int volt = ion.getVoltage();
        int base_color;
        if (volt < 50) {
            base_color = 0x40;
        } else if (volt < 100) {
            base_color = 0x10;
        } else if (volt < 150) {
            base_color = 0x16;
        } else {
            base_color = 0x20;
        }
        return base_color + prob + 2;
    }

    void drawPrediction(GfxContext& ctx, GChartViewport& vp, const GIonStorm& ion, int dh, int warp)
    {
        // chartdlg.pas::DrawStormPrediction
        if (ion.getRadius() < 200) {
            warp = 6;
        }
        if (ion.getVoltage() > 250) {
            warp = 8;
        }
        int h = ion.getHeading();
        GPoint n = ion.getPos();
        for (int j = 0; j < 5; ++j) {
            h += dh;
            n.x += roundToInt(square(warp) * std::sin(h * (PI/180.0)));
            n.y += roundToInt(square(warp) * std::cos(h * (PI/180.0)));
            drawFilledCircle(ctx, vp.scaleXY(n), vp.scale(ion.getRadius()));
        }
    }
}

void
WIonForecastChart::drawPre(GfxCanvas& can)
{
    // chartusr.pas::PredictStorm
    GIonStorm* ion = dynamic_cast<GIonStorm*>(current_ion_selection.getCurrentObject());
    if (!ion) {
        return;
    }

    // Chart viewport
    GChartViewport& vp = getViewport();

    // Build a context
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.setFillPattern(GRAY25_FILL);
    for (int it = 0; it < 6; ++it) {
        ctx.setColor(getPredictionColor(*ion, it));
        drawPrediction(ctx, vp, *ion, -10 + 2*it, 2 + it/2);
        drawPrediction(ctx, vp, *ion, +10 - 2*it, 2 + it/2);
    }
    drawFilledCircle(ctx, vp.scaleXY(ion->getPos()), vp.scale(ion->getRadius()));
}

void
WIonForecastChart::setInitialZoom()
{
    // Initial zoom.
    // PCC 1.x lets initial zoom always start at 1:2, which seems to work well.
    getViewport().mult = 1;
    getViewport().divi = 2;
}

bool
WIonForecastChart::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key && e.key.code == SDLK_TAB) {
        setCenterPosition(getScannerTarget());
        return true;
    }
    return WScannerChartWidget::handleEvent(e, second_pass);
}

/** Ion storm forecast.
    Shows the forecast dialog for the current ion storm. */
void
doIonStormForecast()
{
    // Fetch ion storm
    GIonStorm* ion = dynamic_cast<GIonStorm*>(current_ion_selection.getCurrentObject());
    if (!ion) {
        return;
    }

    // Set origin
    int offset = square(ion->getSpeed()) + ion->getRadius();
    double angle = ion->getHeading() * (PI/180.0);
    GPoint origin = ion->getPos()
        + GPoint(roundToInt(std::sin(angle) * offset),
                 roundToInt(std::cos(angle) * offset));

    // Build window. Modeled largely after doGenericNavigationChart().
    int size = ui_root->getExtent().h;

    // VBox
    //   UIFrameGroup
    //     WIonForecastChart
    //   HBox
    //     WScanResult
    //     VBox
    //       UISpacer
    //       HBox
    //         "OK"
    //         "Help"

    UIWindow win(_("Ion Storm Forecast"), UIWindow::BLUE_WINDOW, COLOR_GRAY);
    win.setLayout(UIVBoxLayout::instance5);

    ObjectHolder h;

    UIGroup& g2   = h.add(new UIGroup(UIHBoxLayout::instance5));
    UIGroup& g22  = h.add(new UIGroup(UIVBoxLayout::instance5));
    UIGroup& g222 = h.add(new UIGroup(UIHBoxLayout::instance5));

    WIonForecastChart& chart = h.add(new WIonForecastChart());
    chart.setCenterPosition(origin);
    chart.setScannerOrigin(origin);
    chart.setScannerTarget(origin);
    chart.setState(chart.sc_StyleChange | chart.sc_Zoomable | chart.sc_Keyboard, true);
    win.add(UIFrameGroup::wrap(h, chart));

    WScanResult& scanner = h.add(new WScanResult(false));
    scanner.setChart(chart);
    g2.add(scanner);
    g2.add(g22);
    g22.add(h.add(new UISpacer()));
    g22.add(g222);

    g222.add(h.add(new UISpacer()));
    g222.add(h.add(new UIButton(1, _("Close"), SDLK_ESCAPE)).addStop());
    g222.add(h.add(new UIButton("H", 'h')));

    win.add(g2);
    win.add(h.add(new WHelpWidget("pcc2:ionforecast")));
    win.add(h.add(new UIPrefixArg()));
    win.add(h.add(new UIQuit(1)));
    win.setExtent(GfxRect(0, 0, size - 50, size - 10));
    chart.setInitialZoom();

    ui_root->centerWidget(win);
    ui_root->runModal(win);
}
