/**
  *  \file client/tiles/nheader.cc
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/nheader.h"
#include "cpluslib/format.h"
#include "game/config.h"
#include "game/planet.h"
#include "game/racename.h"
#include "game/ship.h"
#include "game/spec.h"
#include "game/tables.h"
#include "gfx/palette.h"
#include "resmgr/resid.h"
#include "resmgr/resmgr.h"
#include "ui/uiglobal.h"
#include "ui/util.h"
#include "util/nls.h"
#include "util/misc.h"


WNarrowHeaderTile::WNarrowHeaderTile(GObjectSelection& selection)
    : WObjectSelectionObserverTile(GfxPoint(250, 99), // FIXME?
                              selection)
{ }

void
WNarrowHeaderTile::drawData(GfxCanvas& can)
{
    using ::outTextF;

    /* Picture is 93 pixels, with frame 95, with two pixels space on each side: 99. */
    /*  ###### Name           Name
        ###### (Id #10)       (Id #10)
        ###### 75F, warm      EMERALD
        ###### Tholians       Tholians
        ###### Soldier        Soldier
        ###### */

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());

    GfxRect in = getInnerSize(0, 0);

    // Acquire data
    ResId image;
    string_t name, type, mass;
    int id, owner, level;
    if (GPlanet* pl = dynamic_cast<GPlanet*>(getCurrentObject())) {
        name = pl->getName(GObject::PlainName);
        id = pl->getId();
        if (pl->getTemperature().isKnown()) {
            // C2 B0 = U+00B0 = DEGREE SIGN
            // The "F" is encoded as hex as well because stupid "xgettext 0.19.3" will otherwise interpret the
            // preceding "\xB0" as "\xB0F", which would be equivalent to "\x0F".
            // "xgettext 0.10.40" previously worked.
            type = format(_("%d" "\xC2\xB0" "\x46, %s"), pl->getTemperature(), getTemperatureName(pl->getTemperature()));
            image = ResId(res::planet, pl->getTemperature(), pl->getId());
        } else {
            image = ResId(res::planet);
        }
        owner = pl->hasAnyPlanetData() ? pl->getOwner() : -1;
        level = pl->unit_scores.getScore(planet_score_definitions.lookupScore(ScoreId_ExpLevel));
    } else if (GShip* sh = dynamic_cast<GShip*>(getCurrentObject())) {
        name = sh->getName(GObject::PlainName);
        id = sh->getId();
        mp16_t h = sh->getHullId();
        if (h.isKnown()) {
            type = getHull(h).getName();
            image = ResId(res::ship, getHull(h).getInternalPictureNumber(), h);
        } else {
            type = _("Unknown type");
            image = ResId("nvc");
        }
        // FIXME: guessed position, controller!!!!!!!
        owner = sh->getOwner();
        level = sh->unit_scores.getScore(ship_score_definitions.lookupScore(ScoreId_ExpLevel));
        if (sh->getMass().isKnown()) {
            mass = format(_("%d kt"), numToString(sh->getMass()));
        }
    } else {
        drawBackground(ctx, in);
        return;
    }

    // Draw it
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setSolidBackground();

    drawFrameDown(can, GfxRect(in.x + 2, in.y + 2, 107, 95));

    Ptr<GfxPixmap> pix = ResManager::getInstance().getPixmap(image);
    if (pix)
        pix->blitSized(can, in.x + 3, in.y + 3, 105, 93);
    else
        drawSolidBar(can, GfxRect(in.x + 3, in.y + 3, 105, 93), standard_colors[COLOR_BLACK]);

    // Text
    int x = in.x + 115;
    int y = in.y + 2;
    int w = in.w - 115;
    int dy = font_heights[FONT_NORMAL];

    ctx.setColor(UIColor::tc_White);
    outTextF(ctx, x, y, w, name);
    y += dy;

    ctx.setColor(UIColor::tc_Yellow);
    outTextF(ctx, x, y, w, format(_("(Id #%d)"), id));
    y += dy;

    ctx.setColor(UIColor::tc_Static);
    outTextF(ctx, x, y, w, type);
    y += dy;

    if (owner == 0)
        outTextF(ctx, x, y, w, _("unowned"));
    else if (owner > 0)
        outTextF(ctx, x, y, w, player_racenames.getShortName(owner));
    y += dy;

    if (level >= 0)
        outTextF(ctx, x, y, w, getExperienceLevelName(level));
    y += dy;

    outTextF(ctx, x, y, w, mass);
}
