/**
  *  \file client/tiles/planetnatives.cc
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/planetnatives.h"
#include "game/planet.h"
#include "ui/uiglobal.h"
#include "game/tables.h"
#include "cpluslib/format.h"
#include "util/nls.h"
#include "ui/table.h"
#include "util/misc.h"
#include "ui/util.h"

/*************************** WPlanetNativeTile ***************************/

/** \class WPlanetNativeTile
    \brief Planet Native Tile

    Displays information about a planet's natives, if any. */

WPlanetNativeTile::WPlanetNativeTile(GObjectSelection& selection)
    : WObjectSelectionObserverTile(GfxPoint(315, 4 * font_heights[FONT_NORMAL]), selection)
{ }

void
WPlanetNativeTile::drawData(GfxCanvas& can)
{
    GPlanet* p = dynamic_cast<GPlanet*>(getCurrentObject());
    if (!p || !p->hasFullPlanetData())
        return;

    const int y = font_heights[FONT_NORMAL];

    GfxContext ctx;
    ctx.useCanvas(can).useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setColor(UIColor::tc_Static);

    GfxRect r = getInnerSize(5, 0);
    if (!p->getNativeRace()) {
        drawBackground(ctx, GfxRect(5, 0, r.w, 4*y));
        outTextFormattedF(ctx, r.x, r.y, r.w, 2,
                          _("There are no natives on this planet."));
    } else {
        // This erases the sentence from above because the table
        // writer doesn't (can't?) do that
        drawBackground(ctx, GfxRect(5, 0, r.w, 2*y));

        UITableWriter tw(ctx, r, FONT_NORMAL);

        tw.newLColumn();
        tw.writeFixedCell(UIColor::tc_Static, _("Race:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Population:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Tax Rate:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Natives are"));

        tw.newLColumn(200);
        tw.writeVariableCell(UIColor::tc_Green, string_t(getNativeRaceName(p->getNativeRace())) + ", " + getNativeGovernmentName(p->getNativeGovernment()));
        tw.writeVariableCell(UIColor::tc_Green, clansToString(p->getNatives()));
        tw.writeVariableCell(UIColor::tc_Green, format("%d%%", p->getNativeTax()));

        int happy = p->getNativeHappiness();
        UISkin::Color c = (happy <= 40
                           ? UIColor::tc_Red
                           : happy < 70
                           ? UIColor::tc_Yellow
                           : UIColor::tc_Green);
        tw.writeVariableCell(c, format("%s (%d)", getHappinessName(happy), happy));

        tw.finish();
    }
}



WNarrowPlanetNativeTile::WNarrowPlanetNativeTile(GObjectSelection& selection)
    : WObjectSelectionObserverTile(GfxPoint(250, 4 * font_heights[FONT_NORMAL]), selection)
{ }

void
WNarrowPlanetNativeTile::drawData(GfxCanvas& can)
{
    GPlanet* pl = dynamic_cast<GPlanet*>(getCurrentObject());
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    if (!pl || !(pl->getNativeRace().isKnown() || pl->isKnownToHaveNatives())) {
        drawBackground(ctx, getInnerSize(0, 0));
        return;
    }

    int x = 0;
    int y = 0;
    int w = getInnerSize(0, 0).w;

    /* Anyone on this planet? */
    if (!pl->getNativeRace().isKnown()) {
        /* This means this planet is known to have natives, but we don't know which ones. */
        ctx.setColor(UIColor::tc_Static);
        outTextF(ctx, x, y, w, _("Planet has natives."));
        return;
    }

    if (pl->getNativeRace() == 0) {
        /* No natives. */
        ctx.setColor(UIColor::tc_Static);
        outTextF(ctx, x, y, w, _("No natives."));
        return;
    }

    /* Normal display:
         Race, 10000 clans
         Government
         They like your leadership.
         Taxes: 99% (100 mc) */
    ctx.setColor(UIColor::tc_Green);
    if (pl->getNatives().isKnown()) {
        outTextF(ctx, x, y, w, format(_("%s natives, %d clan%!1{s%}"),
                                      getNativeRaceName(pl->getNativeRace()),
                                      numToString(pl->getNatives())));
    } else {
        outTextF(ctx, x, y, w, getNativeRaceName(pl->getNativeRace()));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getNativeGovernment().isKnown()) {
        ctx.setColor(UIColor::tc_Green);
        outTextF(ctx, x, y, w, getNativeGovernmentName(pl->getNativeGovernment()));
    } else {
        ctx.setColor(UIColor::tc_Static);
        outTextF(ctx, x, y, w, _("Unknown government."));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getNativeHappiness().isKnown()) {
        int happy = pl->getNativeHappiness();
        ctx.setColor(happy <= 40
                     ? UIColor::tc_Red
                     : happy < 70
                     ? UIColor::tc_Yellow
                     : UIColor::tc_Green);
        outTextF(ctx, x, y, w, format("%s (%d)", getHappinessName(happy), happy));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getNativeTax().isKnown()) {
        const char* txt = _("Tax Rate:");
        const int wi = fonts[FONT_NORMAL]->getTextWidth(txt) + font_ems[FONT_NORMAL];
        ctx.setColor(UIColor::tc_Static);
        outText(ctx, x, y, txt);
        ctx.setColor(UIColor::tc_Green);
        outTextF(ctx, x+wi, y, w-wi, format("%d%%", pl->getNativeTax()));
    }
}
