/**
  *  \file client/widgets/expformat.cc
  *  \brief WExportFormatControl widget
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cctype>
#include <algorithm>
#include "client/widgets/expformat.h"
#include "int/export/expdbf.h"
#include "int/export/expjson.h"
#include "int/export/expsep.h"
#include "int/export/exptext.h"
#include "io/textfile.h"
#include "ui/listbox.h"
#include "ui/uiglobal.h"
#include "ui/util.h"
#include "ui/widgets/button.h"
#include "util/nls.h"
#include "util/recode.h"
#include "util/strtab.h"


/** Simple list. Displays a StringTable as a list and lets the user select an item.
    \param title [in] Dialog title
    \param current [in/out] Current value
    \param tab [in] String table
    \retval true User selected a new item; current was updated
    \retval false User canceled */
static bool
doList(const string_t title, int& current, const StringTable& tab)
{
    UIStringList list;
    for (int i = tab.getFirstIndex(); i < tab.getLastIndex(); i = tab.getNextIndex(i)) {
        list.add(i, tab(i));
    }

    UIStandardListbox box(0, list.getCount(), getTextWidth(*fonts[FONT_NORMAL], list) + 20, list);
    box.scrollTo(current, 1);
    if (box.doStandardListbox(title, "pcc2:export")) {
        current = box.getCurrent();
        return true;
    } else {
        return false;
    }
}



static const char*const left_side[] = {
    N_("File type:"),
    N_("Character set:"),
};

static const char*const type_names[] = {
    N_("Text file"),
    N_("Text table"),
    N_("Comma-separated values"),
    N_("Tab-separated values"),
    N_("Semicolon-separated values"),
    N_("JSON (JavaScript)"),
    N_("dBASE file (*.dbf)"),
};

static const char*const type_extensions[] = {
    "txt",
    "txt",
    "csv",
    "csv",
    "csv",
    "js",
    "dbf",
};

namespace {
    /** Charset names as StringTable. */
    class CharsetNames : public StringTable {
        bool longStyle;
     public:
        CharsetNames(bool longStyle)
            : longStyle(longStyle)
            { }
        int getLastIndex() const
            {
                return CharacterSet::getNumCharsets()+1;
            }
        string_t get(int index) const
            {
                if (index == 0) {
                    return _("Unicode (UTF-8)");
                } else {
                    if (longStyle) {
                        return CharacterSet::getCharset(index-1).getDescription();
                    } else {
                        return CharacterSet::getCharset(index-1).getName();
                    }
                }
            }
    };
}

/** Constructor. */
WExportFormatControl::WExportFormatControl()
    : WTile(GfxPoint(1, 1)),
      type(tText),
      charset_index(0)
{
    init();
}

WExportFormatControl::~WExportFormatControl()
{ }

bool
WExportFormatControl::handleEvent(const UIEvent& event, bool second_pass)
{
    if (event.type == ev_Key) {
        switch (event.key.code) {
         case 'f':
            changeType();
            return true;
         case 'c':
            changeCharset();
            return true;
        }
    }
    return WTile::handleEvent(event, second_pass);
}

void
WExportFormatControl::getLayoutInfo(LayoutInfo& info)
{
    info.min_size = info.pref_size = GfxPoint(right_x + 5, countof(left_side) * font_heights[FONT_NORMAL]);
    info.flags = 0;
}

void
WExportFormatControl::drawData(GfxCanvas& can)
{
    const int h = font_heights[FONT_NORMAL];

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);

    /* Left side */
    ctx.setTextAlign(0, 0);
    ctx.setColor(UIColor::tc_Static);
    for (uint_t i = 0; i < countof(left_side); ++i)
        outText(ctx, left_x, h*i, _(left_side[i]));

    /* Right side */
    ctx.setTextAlign(2, 0);
    ctx.setColor(UIColor::tc_Green);
    ctx.setSolidBackground();
    outTextF(ctx, right_x, 0, right_wi, _(type_names[type]));
    outTextF(ctx, right_x, h, right_wi, CharsetNames(false).get(charset_index));
}

/** Perform actual export.
    \param file File to write to
    \param array Array to export
    \param filter Object filter
    \param job Export field list */
void
WExportFormatControl::doExport(Stream& file, IntContext* array, AnswerProvider& filter, IntExportFieldList& job)
{
    if (type == tDBF) {
        /* DBF output */
        IntDbfExporter(file).doExport(array, filter, job);
    } else {
        /* Text output */
        TextFile tf(file);
        if (charset_index == 0) {
            tf.setUtf8();
        } else {
            tf.setCharacterSet(CharacterSet::getCharset(charset_index-1));
        }
        switch (type) {
         case tText:
            IntTextExporter(tf, false).doExport(array, filter, job);
            break;
         case tTable:
            IntTextExporter(tf, true).doExport(array, filter, job);
            break;
         case tCommaSV:
            IntSeparatedTextExporter(tf, ',').doExport(array, filter, job);
            break;
         case tTabSV:
            IntSeparatedTextExporter(tf, '\t').doExport(array, filter, job);
            break;
         case tSemicolonSV:
            IntSeparatedTextExporter(tf, ';').doExport(array, filter, job);
            break;
         case tJSON:
            IntJsonTextExporter(tf).doExport(array, filter, job);
            break;
         case tDBF:
            /* not possible */
            break;
        }
        tf.sync();
    }
}

/** Get recommended extension for selected file format. */
string_t
WExportFormatControl::getRecommendedFileExtension()
{
    return type_extensions[type];
}

void
WExportFormatControl::init()
{
    using std::tolower;
    computeLayout();
    setLayout(LeftJustified, 0, false);

    // Buttons
    int y = 0;
    const int d = font_heights[FONT_NORMAL];
    for (const char* p = "FC"; *p != 0; ++p) {
        UIButton& btn = h.add(new UIButton(string_t(p, 1), tolower(*p)));
        btn.setState(UIButton::bf_SmallSize, true);
        addControl(GfxRect(0, y, d-2, d-2), btn);
        y += d;
    }
}

void
WExportFormatControl::computeLayout()
{
    int lw = getTextWidth(*fonts[FONT_NORMAL], StaticTranslatedStringTable(left_side, countof(left_side)));
    int tw = getTextWidth(*fonts[FONT_NORMAL], StaticTranslatedStringTable(type_names, countof(type_names)));
    int cw = getTextWidth(*fonts[FONT_NORMAL], CharsetNames(false));
    int rw = std::max(tw, cw);

    left_x   = font_heights[FONT_NORMAL] + 5;
    right_x  = left_x + lw + rw;
    right_wi = rw;
}

/** User action: change type. */
void
WExportFormatControl::changeType()
{
    int current = type;
    if (doList(_("Change File Type"), current, StaticTranslatedStringTable(type_names, countof(type_names)))) {
        type = Type(current);
        drawWidget();
    }
}

/** User action: change charset. */
void
WExportFormatControl::changeCharset()
{
    int current = charset_index;
    if (doList(_("Change Character Set"), current, CharsetNames(true))) {
        charset_index = current;
        drawWidget();
    }
}
