/**
  *  \file flak/vis-sprites.cc
  *  \brief Simple Visualisation Sprites
  *
  *  These sprites are shared between different visualisation modes.
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "flak/vis-sprites.h"
#include "gfx/palette.h"

/***************************** FlakExplSprite ****************************/

/*! \class FlakExplSprite
    This sprite displays an explosion. */

/** Constructor.
    \param p world position
    \param vis visualizer this is displayed in
    \param size size of explosion
    \param speed speed */
FlakExplSprite::FlakExplSprite(FlakPos p, FlakFancyVisualizer& vis, int size, int speed)
    : VcrExplSprite(GfxPoint(0, 0), 0, size, speed), vis(vis), p(p)
{
    setId(vis.layer_Bang);
    tick();
    vis.addExplosionSprite();
}

FlakExplSprite::~FlakExplSprite()
{
    vis.removeExplosionSprite();
}

void
FlakExplSprite::tick()
{
    if (vis.isTimeAdvance())
        VcrExplSprite::tick();

    FlakScreenPos sp = vis.getScreenPosition(p);
    setCenterPos(sp, sp.z);
}

/************************* FlakSimpleObjectSprite ************************/

/*! \class FlakSimpleObjectSprite
    This sprite displays a simple rendering of an object, that is,
    a torpedo (red cross) or fighter (blue circle). */
FlakSimpleObjectSprite::~FlakSimpleObjectSprite()
{ }

/** Create sprite for one object.
    \todo The sprite is not yet added to the visualizer. Maybe it should?
    \param vis target visualizer
    \param obj object
    \return new sprite */
FlakSimpleObjectSprite*
FlakSimpleObjectSprite::create(FlakFancyVisualizer& vis, FlakObject& obj)
{
    FlakScreenPos p = vis.getScreenPosition(obj.position);
    const int r = obj.kind == obj.oTorpedo ? TORP_RADIUS : FIGHTER_RADIUS;

    return new FlakSimpleObjectSprite(FlakScreenArea(p.x - r, p.y - r,
                                                     2*r+1, 2*r+1,
                                                     p.z),
                                      vis, obj);
}

void
FlakSimpleObjectSprite::draw(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    if (obj.kind == obj.oFighter) {
        ctx.setColor(COLOR_BLUE);
        drawCircle(ctx, GfxPoint(getPosCX(), getPosCY()), FIGHTER_RADIUS);
    } else {
        ctx.setColor(COLOR_RED);
        drawHLine(ctx, getPos().x, getPosCY(), getPos().x + getPos().w - 1);
        drawVLine(ctx, getPosCX(), getPos().y, getPos().y + getPos().h - 1);
    }
}

void
FlakSimpleObjectSprite::tick()
{
    setCenterPos2D(vis.getScreenPosition(obj.position));
}

/***************************** FlakBeamSprite ****************************/

/*! \class FlakBeamSprite
    This sprite represents a beam. It lives for 6 ticks, and displays sort-of
    a moving line. */

/** Constructor.
    \param start originator of shot
    \param end target of shot
    \param vis visualizer this is displayed on */
FlakBeamSprite::FlakBeamSprite(FlakPos start, FlakPos end, FlakFancyVisualizer& vis)
    : GfxSprite3D(vis.layer_Beam), start(start), end(end), vis(vis), status(0)
{
    vis.addEffectSprite();
    tick();
}

FlakBeamSprite::~FlakBeamSprite()
{
    vis.removeEffectSprite();
}

void
FlakBeamSprite::tick()
{
    // advance time
    if (vis.isTimeAdvance()) {
        ++status;
        if (status >= 6) {
            deleteThis();
            return;
        }
        update();
    }

    // compute position
    spos = vis.getScreenPosition(start);
    epos = vis.getScreenPosition(end);

    setPos(GfxRect(std::min(spos.x, epos.x), std::min(spos.y, epos.y),
                   std::abs(spos.x - epos.x)+1, std::abs(spos.y - epos.y) + 1),
           (spos.z + epos.z) >> 1);
}

void
FlakBeamSprite::draw(GfxCanvas& can)
{
    static const int colors[6][3] = {
        { COLOR_WHITE, COLOR_DARKCYAN, COLOR_DARKCYAN },
        { COLOR_WHITE, COLOR_WHITE, COLOR_DARKCYAN },
        { COLOR_WHITE, COLOR_WHITE, COLOR_WHITE },
        { COLOR_BLUE, COLOR_WHITE, COLOR_WHITE },
        { COLOR_BLUE, COLOR_BLUE, COLOR_WHITE },
        { COLOR_BLUE, COLOR_BLUE, COLOR_BLUE },
    };

    const int x0 = spos.x, y0 = spos.y, x1 = epos.x, y1 = epos.y;
    const int x2 = (2*x0+x1)/3;
    const int y2 = (2*y0+y1)/3;
    const int x3 = (2*x1+x0)/3;
    const int y3 = (2*y1+y0)/3;

    ASSERT(status >= 0 && status < 6);
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.setColor(colors[status][0]);
    drawLine(ctx, GfxPoint(x0, y0), GfxPoint(x2, y2));
    ctx.setColor(colors[status][1]);
    drawLine(ctx, GfxPoint(x2, y2), GfxPoint(x3, y3));
    ctx.setColor(colors[status][2]);
    drawLine(ctx, GfxPoint(x3, y3), GfxPoint(x1, y1));
}

/************************** FlakSimpleShipSprite *************************/

/*! \class FlakSimpleShipSprite
    This displays a ship as a simple circle. */

void
FlakSimpleShipSprite::tick()
{
    FlakScreenPos p = vis.getScreenPosition(vis.getBattle().getShipByNumber(ship).getPos());
    setCenterPos(p, p.z);
}

void
FlakSimpleShipSprite::draw(GfxCanvas& can)
{
    const FlakShip& sh = vis.getBattle().getShipByNumber(ship);
    if (sh.isAlive()) {
        GfxContext ctx;
        ctx.useCanvas(can);
        ctx.useColorScheme(GfxStandardColorScheme::instance);
        if (sh.isPlanet()) {
            ctx.setColor(COLOR_BLUE);
            drawFilledCircle(ctx, GfxPoint(getPosCX(), getPosCY()), SHIP_RADIUS);
        } else {
            ctx.setColor(COLOR_BLUE);
            drawFilledCircle(ctx, GfxPoint(getPosCX(), getPosCY()), SHIP_RADIUS);
            ctx.setColor(COLOR_WHITE);
            drawCircle(ctx, GfxPoint(getPosCX(), getPosCY()), SHIP_RADIUS);
        }
    }
}
