/**
  *  \file game/objsel.h
  *  \brief Object Selection
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_OBJSEL_H
#define PCC_V2_GAME_OBJSEL_H

#include "cpluslib/sigslot.h"

class GObject;
class GUniverse;
class GObjectType;

/** Object selection, base class. This provides functions to manage
    selection of an single object from a set (GObjectType), used to
    denote the "current" object in a context. */
class GObjectSelection {
 public:
    // Derived-class Interface:
    virtual ~GObjectSelection();

    /** Get underlying object type. This is used to find out possible
        indexes to select. Can be null. */
    virtual GObjectType* getObjectType() const = 0;

    /** Set current index.
        \pre index==0 || getObjectType()->isValidIndex(index) */
    virtual void setCurrentIndex(int index) = 0;

    /** Get currently-selected index. Nonzero if it is valid, zero
        if no object is selected. */
    virtual int getCurrentIndex() const = 0;

    // GObjectSelection:
    GObject* getCurrentObject() const;
    GUniverse* getCurrentUniverse() const;

    Signal0 sig_current_changed;
};

/** Object selection, simple version. This implements the simple
    way of selecting an object, by tracking a current index.
    This is the sufficient solution for most places.

    Structural changes are handled by trying to switch to the object
    given as a hint in the sig_set_changed event, which is the right
    thing for fleet renamings. For other objects that go away there
    probably is no unique "right" way.

    This object can be told to point at any GObjectType. When not
    pointing at one, it correctly reports no object selected. */
class GSimpleObjectSelection : public GObjectSelection {
 public:
    GSimpleObjectSelection();
    ~GSimpleObjectSelection();

    // GObjectSelection:
    GObjectType* getObjectType() const;
    void setCurrentIndex(int index);
    int getCurrentIndex() const;

    // GSimpleObjectSelection:
    void setObjectType(GObjectType* type);
    void initFrom(GObjectSelection& other);

 private:
    int current;
    GObjectType* type;

    SignalConnection conn_set_changed;
    void onSetChanged(int hint);
};

/** Proxy object selection. This implements all methods by referring
    them to another instance of a GObjectSelection which can be
    configured at runtime. */
class GProxyObjectSelection : public GObjectSelection {
 public:
    GProxyObjectSelection();
    ~GProxyObjectSelection();

    // GObjectSelection:
    GObjectType* getObjectType() const;
    void setCurrentIndex(int index);
    int getCurrentIndex() const;

    // GProxyObjectSelection:
    void setObjectSelection(GObjectSelection* selection);

 private:
    GObjectSelection* selection;
    SignalConnection conn_current_changed;
};

#endif
