/**
  *  \file game/parser.cc
  *  \brief Message Parser Data Structures
  *
  *  In addition to information produced by regular data files (PDATA,
  *  VCR, etc.), information comes in mixed ad-hoc forms. This
  *  information is converted into a uniform structure before being
  *  assimilated by the particular components:
  *
  *  - each message produces one or more GMessageInformation's
  *  - each GMessageInformation contains information about a single
  *    target object
  *  - each GMessageInformation can contain multiple values for
  *    that target object
  *  - each GMessageInformation can contain information about a
  *    single turn
  *
  *  Messages can also produce empty GMessageInformation to just
  *  relate the message to an object.
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/parser.h"
#include "util/misc.h"

/** Constructor.
    \param obj target object type
    \param id target object Id, if appropriate for its type
    \param turn turn this information is from */
GMessageInformation::GMessageInformation(GMessageObject obj, int32_t id, int turn)
    : obj(obj), id(id), values(), turn(turn)
{ }

GMessageInformation::~GMessageInformation()
{ }

/** Add string value. */
void
GMessageInformation::addValue(GMessageStringIndex si, const string_t& s)
{
    values.push_back(new GMessageStringValue(si, s));
}

/** Add integer value. */
void
GMessageInformation::addValue(GMessageIntIndex ii, int32_t i)
{
    values.push_back(new GMessageIntValue(ii, i));
}

/** Add configuration key. For MessageConfig only. */
void
GMessageInformation::addConfigValue(string_t key, string_t value)
{
    ASSERT(obj == MessageConfig);
    values.push_back(new GMessageConfigValue(key, value));
}

/** Add player score. For MessagePlayerScore only. */
void
GMessageInformation::addScoreValue(int player, int32_t value)
{
    ASSERT(obj == MessagePlayerScore);
    values.push_back(new GMessageScoreValue(player, value));
}

/** Add alliance value. For MessageAlliance only. */
void
GMessageInformation::addAllianceValue(string_t id, const GAllianceOffer& offer)
{
    ASSERT(obj == MessageAlliance);
    values.push_back(new GMessageAllianceValue(id, offer));
}
