/**
  *  \file game/team.cc
  *  \brief Players and Teams
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <cstring>
#include "game/team.h"
#include "game/reg.h"
#include "io/streams.h"
#include "game/gamedirs.h"
#include "cpluslib/format.h"
#include "util/nls.h"
#include "game/playerset.h"
#include "game/struct.h"
#include "util/except.h"
#include "io/io.h"
#include "cpluslib/sigslot.h"
#include "util/modinit.h"
#include "util/recode.h"
#include "game/alliance.h"
#include "game/pref.h"

namespace {
    struct TeamInfo {
        uint16_t flags;
        GPlayerArray<int> player_teams;
        GPlayerArray<string_t> team_names;
        GPlayerArray<int> send_config;
        GPlayerArray<int> receive_config;
        uint16_t passcode;
    };
}

static const char TEAM_MAGIC[] = { 'C', 'C', 't', 'e', 'a', 'm', '0', 26 };

/**************************** Player Id Status ***************************/

/** Current player id. The client presents the game status from this
    player's point of view. \see getPlayerId() */
static int player_id;

/** Current real player id. This is the number of the player we're
    authenticated as. \see getRealPlayerId() */
static int real_player_id;

/** Current team definitions. */
static TeamInfo team_info;

/** Signal for team change. Raised whenever anything related to teams is changed. */
Signal0 sig_team_change;

/** Signal for player change. Raised whenever current player selection changes. */
Signal0 sig_player_change;

/***************************** Load and Save *****************************/

static void
doneTeams()
{
    team_info.flags = 0;
    for (int i = 0; i <= NUM_OWNERS; ++i)
        team_info.player_teams[i] = i;
    team_info.team_names.setAll(string_t());
    team_info.send_config.setAll(0);
    team_info.receive_config.setAll(0);
}

static void
loadTeams(Stream& s)
{
    /* File starts with
       +0    8 bytes magic
       +8    2 bytes flags
       +10  12 bytes team numbers 1..12
       +22  12 bytes team colors (unused)
       +34 total */
    char buffer[34];
    s.readT(buffer, sizeof(buffer));
    if (std::memcmp(buffer, TEAM_MAGIC, sizeof(TEAM_MAGIC)) != 0)
        throw FileFormatException(s, _("File is missing required signature"));

    /* Remember header */
    team_info.flags = getUint16(buffer+8);
    for (int i = 1; i <= 12; ++i)
        if (buffer[i+9] > 0 && buffer[i+9] <= 12)
            team_info.player_teams[i] = buffer[i+9];

    /* Read names */
    for (int i = 1; i <= 12; ++i) {
        string_t tmp;
        getPascalString(s, tmp);        /* silently ignore file truncation */
        team_info.team_names[i] = convertGameToUtf8(tmp);
    }

    /* Read data transfer settings:
       11 bytes send
       11 bytes receive
       2 bytes passcode */
    if (s.read(buffer, 24) == 24) {
        for (int i = 1; i <= 11; ++i) {
            team_info.send_config[i] = buffer[i-1];
            team_info.receive_config[i] = buffer[i-1+11];
        }
        team_info.passcode = getUint16(buffer+22);
    }

    sig_team_change.raise();
}

static void
saveTeams(Stream& s)
{
    /* Header */
    char buffer[34];
    std::memcpy(buffer, TEAM_MAGIC, sizeof(TEAM_MAGIC));
    storeUint16(buffer+8, team_info.flags);
    for (int i = 1; i <= 12; ++i) {
        /* Fill in team assignments. Team colors are not used by
           anything, so fill in some defaults (similar to PCC 1.x) */
        buffer[i+9]  = team_info.player_teams[i];
        buffer[i+21] = (team_info.player_teams[i] == team_info.player_teams[::getPlayerId()] ? 3 : 4);
    }
    s.writeT(buffer, sizeof(buffer));

    /* Names */
    for (int i = 1; i <= 12; ++i)
        storePascalStringT(s, convertUtf8ToGame(team_info.team_names[i]));

    /* Data transfer */
    for (int i = 1; i <= 11; ++i) {
        buffer[i-1] = team_info.send_config[i];
        buffer[i-1+11] = team_info.receive_config[i];
    }
    storeUint16(buffer+22, team_info.passcode);
    s.writeT(buffer, 24);
}

static void
saveTeams()
{
    if (Ptr<Stream> ps = game_file_dir->openFileNT(format("team%d.cc", getRealPlayerId()), Stream::C_CREATE))
        saveTeams(*ps);
}

static void
initTeams()
{
    // Set defaults
    doneTeams();

    // Load config file if it exists
    if (Ptr<Stream> ps = game_file_dir->openFileNT(format("team%d.cc", getRealPlayerId()), Stream::C_READ))
        loadTeams(*ps);
}

/******************************* Functions *******************************/

/** Get current player. That's the one whose data we're looking at,
    0 if none. */
int
getPlayerId()
{
    return player_id;
}

/** Get current authenticated player. That's the one whose password
    we have. May be different from current player if he used a password
    cookie to become somebody else. */
int
getRealPlayerId()
{
    return real_player_id;
}

/** Set player Ids.
    \param eff   "effective" id. The one in whose name we're giving commands.
    \param real  "real" id. The one which was authenticated. */
void
setPlayerIds(int eff, int real)
{
    real_player_id = real;
    player_id = eff;
    sig_player_change.raise();
}

/** Get relation to player n. */
GPlayerRelation
getPlayerRelation(int n)
{
    if (n == player_id)
        return is_Me;
    else if (team_info.player_teams[n] == team_info.player_teams[player_id])
        return is_Ally;
    else
        return is_Enemy;
}

/** Get number of team a player is in. */
int
getPlayerTeam(int player)
{
    return team_info.player_teams[player];
}

/** Change number of team a player is in. */
void
setPlayerTeam(int player, int team)
{
    if (team != team_info.player_teams[player]) {
        team_info.player_teams[player] = team;
        sig_team_change.raise();
    }
}

/** Remove player from his team. Moves him into a team of his own. */
void
removePlayerTeam(int player)
{
    if (getNumTeamMembers(getPlayerTeam(player)) > 1) {
        if (getNumTeamMembers(player) == 0) {
            // We can put him into the team which has the same number as the player
            setPlayerTeam(player, player);
        } else {
            // Search for an unused team
            for (int i = 1; i <= NUM_PLAYERS; ++i) {
                if (getNumTeamMembers(i) == 0) {
                    setPlayerTeam(player, i);
                    break;
                }
            }
        }
    }
}

/** Get number of team members in a team. */
int
getNumTeamMembers(int team)
{
    int result = 0;
    for (int i = 1; i <= NUM_PLAYERS; ++i)
        if (team_info.player_teams[i] == team)
            ++result;
    return result;
}

/** Get name of a team. */
string_t
getTeamName(int team)
{
    if (team_info.team_names[team].size())
        return team_info.team_names[team];
    else
        return string_t(format(_("Team %d"), team));
}

/** Set name of a team. */
void
setTeamName(int team, string_t name)
{
    if (team_info.team_names[team] != name) {
        team_info.team_names[team] = name;
        sig_team_change.raise();
    }
}

/** Check whether team has a name. If it has not, getTeamName() will return a default name. */
bool
isNamedTeam(int team)
{
    return team_info.team_names[team].size() != 0;
}

/** Check whether there's any team configured. */
bool
isTeamConfigured()
{
    for (int i = 1; i <= NUM_PLAYERS; ++i)
        if (team_info.player_teams[i] != i || team_info.team_names[i].size() != 0)
            return true;
    return false;
}

/** Synchronize teams from alliances. */
void
syncTeamsFromAlliances(const GAlliances& allies)
{
    const int me = getRealPlayerId();
    const int myTeam = getPlayerTeam(me);
    if (getUserPreferences().TeamAutoSync() && allies.getLevels().size() > 0) {
        // We actually have alliances
        for (int i = 1; i <= NUM_PLAYERS; ++i) {
            if (i != me) {
                // It's a relation to another player
                if (getPlayerTeam(i) == myTeam) {
                    // They are on my team. Are we allied? If not, remove them.
                    // (only check our offers; if we offer an alliance, we consider them on our team.)
                    if (!allies.isAny(i, GAllianceLevel::IsOffer, true)) {
                        removePlayerTeam(i);
                    }
                } else {
                    // They are not on my team.
                    if (allies.isAny(i, GAllianceLevel::IsOffer, true)) {
                        setPlayerTeam(i, myTeam);
                    }
                }
            }
        }
    }
}

extern const ModuleInitializer mod_team[] = {
    // Whenever we enter a state where we don't have a player, discard teams
    { OnEnterDirectory | OnEnterNoGame | OnLeftGame, doneTeams },
    // Whenever we enter a state where we have a player, load teams
    { OnEnterGame, initTeams },
    // Whenever we leave a state where we had a player, save teams
    { OnLeaveGame, saveTeams },
    { 0, 0 },
};
