/**
  *  \file gfx/gfxcontext.h
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GFX_GFXCONTEXT_H
#define PCC_V2_GFX_GFXCONTEXT_H

#include "cpluslib/types.h"
#include "gfx/gfxutil.h"
#include "cpluslib/object.h"

class GfxContext;
class GfxFont;
class GfxCanvas;

class GfxColorScheme : public Object {
 public:
    GfxColorScheme()
        { }
    virtual ~GfxColorScheme();

    virtual uint32_t getColor(uint32_t index) = 0;
    virtual void drawBackground(GfxContext& ctx, GfxRect area) = 0;
};

class GfxNullColorScheme : public GfxColorScheme {
 public:
    uint32_t getColor(uint32_t index);
    void drawBackground(GfxContext& ctx, GfxRect area);

    static GfxNullColorScheme instance;
};

class GfxContext {
    uint32_t raw_color;
    uint32_t line_thickness;
    uint8_t  line_pattern;
    bool transparent_background;
    GfxFillPattern fill_pattern;
    alpha_type alpha;
    GfxPoint cursor;
    GfxPoint text_align;
    GfxFont* font;
    GfxCanvas* canvas;
    GfxColorScheme* color_scheme;

 public:
    GfxContext();
    // ~GfxContext();

    GfxContext& setColor(uint32_t color);
    GfxContext& setRawColor(uint32_t color);
    GfxContext& setSolidBackground();
    GfxContext& setTransparentBackground();
    GfxContext& setLineThickness(int n);
    GfxContext& setLinePattern(uint8_t pat);
    GfxContext& setFillPattern(const GfxFillPattern& pat);
    GfxContext& setAlpha(alpha_type alpha);
    GfxContext& setCursor(GfxPoint pt);
    GfxContext& setTextAlign(int x, int y);
    GfxContext& useFont(GfxFont& font);
    GfxContext& useCanvas(GfxCanvas& canvas);
    GfxContext& useColorScheme(GfxColorScheme& color_scheme);

    uint32_t              getRawColor() const;
    bool                  isTransparentBackground() const;
    uint32_t              getLineThickness() const;
    uint8_t               getLinePattern() const;
    GfxFillPattern&       getFillPattern();
    const GfxFillPattern& getFillPattern() const;
    alpha_type            getAlpha() const;
    GfxPoint              getCursor() const;
    GfxPoint              getTextAlign() const;
    GfxFont&              getFont() const;
    GfxCanvas&            getCanvas() const;
    GfxColorScheme&       getColorScheme() const;
};



inline GfxContext&
GfxContext::setRawColor(uint32_t color)
{
    this->raw_color = color;
    return *this;
}

inline GfxContext&
GfxContext::setSolidBackground()
{
    this->transparent_background = false;
    return *this;
}

inline GfxContext&
GfxContext::setTransparentBackground()
{
    this->transparent_background = true;
    return *this;
}

inline GfxContext&
GfxContext::setLineThickness(int n)
{
    this->line_thickness = n;
    return *this;
}

inline GfxContext&
GfxContext::setLinePattern(uint8_t pat)
{
    this->line_pattern = pat;
    return *this;
}

inline GfxContext&
GfxContext::setAlpha(alpha_type alpha)
{
    this->alpha = alpha;
    return *this;
}

inline GfxContext&
GfxContext::setCursor(GfxPoint pt)
{
    this->cursor = pt;
    return *this;
}

inline GfxContext&
GfxContext::setTextAlign(int x, int y)
{
    this->text_align.x = x;
    this->text_align.y = y;
    return *this;
}

inline GfxContext&
GfxContext::useFont(GfxFont& font)
{
    this->font = &font;
    return *this;
}

inline GfxContext&
GfxContext::useCanvas(GfxCanvas& canvas)
{
    this->canvas = &canvas;
    return *this;
}

inline GfxContext&
GfxContext::useColorScheme(GfxColorScheme& color_scheme)
{
    this->color_scheme = &color_scheme;
    return *this;
}

inline uint32_t
GfxContext::getRawColor() const
{
    return raw_color;
}

inline bool
GfxContext::isTransparentBackground() const
{
    return transparent_background;
}

inline uint32_t
GfxContext::getLineThickness() const
{
    return line_thickness;
}
inline uint8_t
GfxContext::getLinePattern() const
{
    return line_pattern;
}
inline GfxFillPattern&
GfxContext::getFillPattern()
{
    return fill_pattern;
}

inline const GfxFillPattern&
GfxContext::getFillPattern() const
{
    return fill_pattern;
}

inline alpha_type
GfxContext::getAlpha() const
{
    return alpha;
}
inline GfxPoint
GfxContext::getCursor() const
{
    return cursor;
}
inline GfxPoint
GfxContext::getTextAlign() const
{
    return text_align;
}
inline GfxFont&
GfxContext::getFont() const
{
    return *font;
}
inline GfxCanvas&
GfxContext::getCanvas() const
{
    return *canvas;
}

inline GfxColorScheme&
GfxContext::getColorScheme() const
{
    return *color_scheme;
}

#endif
