/**
  *  \file int/compound.cc
  *  \brief Interpreter: Compound Values
  *
  *  (c) 2001-2017 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/compound.h"
#include "int/interror.h"
#include "int/context.h"
#include "int/args.h"
#include "int/exec.h"


bool
IntIndexableValue::isProcedureCall()
{
    return false;
}

void
IntIndexableValue::call(IntExecutionContext& exc, IntDataSegment& args, bool want_result)
{
    IntArgBlock a(args, 0, args.size());
    IntValue* v = get(a);
    if (want_result) {
        exc.pushNewValue(v);
    } else {
        delete v;
    }
}


/************************ IntSimpleIndexableValue ************************/

/** Constructor.
    \param get  Function to implement get() (=indexed read access, function call).
                Can be null, in this case read access ('pushind' opcode) will not
                be permitted.
    \param make Function to implement makeFirstContext() (=iteration access).
                Can be null, in this case iteration ('firstindex' opcode) will not
                be permitted. */
IntSimpleIndexableValue::IntSimpleIndexableValue(dim_t dim, get_t get, make_t make)
    : dim_function(dim),
      get_function(get),
      make_function(make)
{ }

IntSimpleIndexableValue::~IntSimpleIndexableValue()
{ }

// IntIndexableValue:
IntValue*
IntSimpleIndexableValue::get(IntArgBlock& args)
{
    if (get_function) {
        return get_function(args);
    } else {
        throw IntError::typeError(IntError::tyIndexable);
    }
}

void
IntSimpleIndexableValue::set(IntArgBlock& /*args*/, IntValue* /*value*/)
{
    throw IntError::notAssignable();
}

int32_t
IntSimpleIndexableValue::getDimension(int dim)
{
    if (dim_function == 0) {
        return 0;
    } else {
        return dim_function(dim);
    }
}

IntContext*
IntSimpleIndexableValue::makeFirstContext()
{
    if (make_function)
        return make_function();
    else
        throw IntError::typeError(IntError::tyIterable);
}

// IntValue:
string_t
IntSimpleIndexableValue::toString(bool /*readable*/)
{
    return "#<array>";
}

void
IntSimpleIndexableValue::store(IntTagNode&, Stream&)
{
    throw IntError::notSerializable();
}

IntValue*
IntSimpleIndexableValue::clone()
{
    return new IntSimpleIndexableValue(*this);
}

/************************ IntSimpleProcedureValue ************************/

IntSimpleProcedureValue::IntSimpleProcedureValue(call_t call)
    : call_function(call)
{ }

IntSimpleProcedureValue::~IntSimpleProcedureValue()
{ }

// IntCallableValue:
void
IntSimpleProcedureValue::call(IntExecutionContext& exc, IntDataSegment& args, bool want_result)
{
    IntArgBlock a(args, 0, args.size());
    call_function(exc, a);
    if (want_result) {
        exc.pushNewValue(0);
    }
}

bool
IntSimpleProcedureValue::isProcedureCall()
{
    return true;
}

int32_t
IntSimpleProcedureValue::getDimension(int32_t)
{
    return 0;
}

IntContext*
IntSimpleProcedureValue::makeFirstContext()
{
    throw IntError::typeError(IntError::tyIterable);
}

// IntValue:
string_t
IntSimpleProcedureValue::toString(bool /*readable*/)
{
    return "#<procedure>";
}

void
IntSimpleProcedureValue::store(IntTagNode&, Stream&)
{
    throw IntError::notSerializable();
}

IntValue*
IntSimpleProcedureValue::clone()
{
    return new IntSimpleProcedureValue(call_function);
}

/******************************* IntClosure ******************************/

IntClosure::IntClosure()
    : function(),
      fixed_args(new IntDataSegment())
{ }

IntClosure::IntClosure(const IntClosure& other)
    : IntCallableValue(*this),
      function(other.function),
      fixed_args(other.fixed_args)
{ }

IntClosure::~IntClosure()
{ }

// IntClosure:
void
IntClosure::setNewFunction(IntCallableValue* function)
{
    this->function = function;
}

void
IntClosure::addNewArgument(IntValue* value)
{
    fixed_args->pushNew(value);
}

void
IntClosure::addNewArgumentsFrom(IntDataSegment& seg, uint32_t nargs)
{
    seg.transferTo(nargs, *fixed_args);
}

// IntCallableValue:
void
IntClosure::call(IntExecutionContext& exc, IntDataSegment& args, bool want_result)
{
    IntDataSegment combined_args;

    // We must not modify fixed_args
    for (uint32_t i = 0, n = fixed_args->size(); i < n; ++i) {
        combined_args.push((*fixed_args)[i]);
    }

    // We can loot args
    args.transferTo(args.size(), combined_args);

    // Call function
    function->call(exc, combined_args, want_result);
}

bool
IntClosure::isProcedureCall()
{
    return function->isProcedureCall();
}

int32_t
IntClosure::getDimension(int32_t which)
{
    int32_t totalDimensions = function->getDimension(0);
    int32_t fixedDimensions = fixed_args->size();
    if (fixedDimensions >= totalDimensions) {
        // All arguments fixed; no way this has any dimensions
        return 0;
    } else {
        // Some dimensions still free
        if (which == 0) {
            // Number of dimensions
            return totalDimensions - fixedDimensions;
        } else if (which + fixedDimensions <= totalDimensions) {
            // Asking for size of an existing dimension
            return function->getDimension(which + fixedDimensions);
        } else {
            // Asking for size of a nonexistant dimension
            return 0;
        }
    }
}

IntContext*
IntClosure::makeFirstContext()
{
    // Since we are providing a "slice" of the array, we cannot make a first context.
    // It would have to represent that slice. Therefore, pretend to be not iterable.
    throw IntError::typeError(IntError::tyIterable);
}

string_t
IntClosure::toString(bool /*readable*/)
{
    return "#<closure>";
}

void
IntClosure::store(IntTagNode&, Stream&)
{
    throw IntError::notSerializable();
}

IntValue*
IntClosure::clone()
{
    return new IntClosure(*this);
}
